#!/usr/bin/env python
# coding: utf-8

import email
import imaplib
import json
import os
import random
import re
import secrets
import shutil
import signal
import string
import sys
import time
from datetime import datetime, timedelta
from pathlib import Path

import factory
import pymongo
from bs4 import BeautifulSoup
from dotenv import load_dotenv
from loguru import logger
from rich.console import Console
from selenium import webdriver
from selenium.common.exceptions import ElementNotInteractableException
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.support.ui import WebDriverWait
from tqdm import tqdm
from twocaptcha import TwoCaptcha  # noqa

from reddit_gen.experimental import vpn_driver
from reddit_gen.handlers import keyboard_interrupt_handler, alarm_handler
from reddit_gen.reddit_settings import RedditSettings
from reddit_gen.utils import check_shadowban, custom_theme, mongodb_client


def _gen_username():
    while True:
        g_username = factory.build(dict,
                                   user=factory.Faker('user_name'))['user']
        rand_int = random.randint(0, 100)
        g_username = f'{g_username}_{rand_int}'
        if len(g_username) < 20:
            return g_username


def _gen_pass():
    punctuation = [
        x for x in list(string.punctuation) if x not in ['"', "'", '\\']
    ]
    punc = ''.join(random.sample(punctuation, 4))
    token = list(secrets.token_urlsafe(12) + punc)
    token = ''.join(random.sample(token, len(token)))
    return token


def _signup_info():
    _username = _gen_username()
    if os.getenv('CATCH_ALL_DOMAIN'):
        _email = f'{_username}@{os.environ["CATCH_ALL_DOMAIN"]}'
    else:
        e_address = os.environ['EMAIL_ADDRESS'].split('@')
        _email = f'{e_address[0]}+{_username}@{e_address[1]}'
    _passwd = _gen_pass()
    logger.opt(colors=True).info(
        f'Your account\'s email address: <u><y>{_email}</y></u>')
    logger.opt(colors=True).info(f'Username: <y>{_username}</y>')
    return _email, _username, _passwd


def _get_verf_url():
    if 'gmail' in os.environ['EMAIL_ADDRESS']:
        os.environ['IMAP_SERVER'] = 'imap.gmail.com'
    mail = imaplib.IMAP4_SSL(os.environ['IMAP_SERVER'])
    mail.login(os.environ['EMAIL_ADDRESS'], os.environ['EMAIL_PASSWD'])
    mail.select('inbox')

    mail_ids = sum([x.split() for x in mail.search(None, 'ALL')[1]], [])
    _, data = mail.fetch(mail_ids[-1], '(RFC822)')

    msg = email.message_from_bytes(data[0][1])  # noqa
    d = {
        'from': msg['from'],
        'subject': msg['subject'],
        'body': msg.get_payload(decode=True)
    }

    soup = BeautifulSoup(d['body'], 'html.parser')
    for link in soup.find_all('a', href=True):
        if '/verification/' in link['href']:
            return link['href']


def check_driver_path():
    driver_path = shutil.which('chromedriver')
    if not driver_path:
        if os.getenv('CHROME_DRIVER_PATH'):
            driver_path = os.environ['CHROME_DRIVER_PATH']
        else:
            print('Cannot find chromedriver! Add the chrome driver path '
                  'manually to the `.env` file.')
    return driver_path


def _cooldown_func(time_left, solve_manually=False):
    signal.signal(signal.SIGALRM, alarm_handler)
    signal.alarm(30)
    try:
        logger.warning('You don\'t need to respond if not applicable. '
                       'The program will automatically fallback to cooldown.')
        if not solve_manually:
            ans = input('Have you changed your IP address? (y/n) ')
            if ans.lower() != 'y':
                signal.alarm(0)
                logger.warning(
                    'You need to wait 10 minutes between new accounts...')
                logger.warning('DO NOT close this.  The program will continue '
                               'when the cooldown period has passed.')
                for _ in tqdm(range(time_left + 5)):
                    time.sleep(1)
            else:
                signal.alarm(0)
    except TimeoutError:
        logger.info('Started cooldown period.')
        for _ in tqdm(range(time_left - 25)):
            time.sleep(1)


def load_driver(driver_path,
                disable_headless=True,
                experimental_use_vpn=False,
                solve_manually=False,
                binary_location=None):
    options = webdriver.ChromeOptions()
    options.binary_location = binary_location
    if not disable_headless and not solve_manually:
        options.add_argument('headless')

    if experimental_use_vpn:
        del options
        try:
            driver = vpn_driver(driver_path, disable_headless=disable_headless)
        except Exception as e:
            logger.exception(e)
            driver.quit()
            sys.exit(1)
    else:
        service = Service(driver_path)
        driver = webdriver.Chrome(options=options, service=service)
    return driver


def generate(driver,
             disable_headless=False,
             solve_manually=False,
             ip_rotated=False,
             use_json=False,
             debug=False,
             experimental_use_vpn=False,
             env_file=None):
    signal.signal(signal.SIGINT, keyboard_interrupt_handler)
    load_dotenv(env_file)

    if not os.getenv('MONGODB_CONNECTION_STRING') and not use_json:
        logger.warning(
            'The environment variable `MONGODB_CONNECTION_STRING` is not '
            'set. Defaulting to a local JSON database.')
        use_json = True

    local_db = f'{Path.home()}/.reddit_accounts.json'
    if use_json and not Path(local_db).exists():
        with open(local_db, 'w') as j:
            json.dump([], j)

    console = Console(theme=custom_theme())

    timestamp = datetime.now()

    driver_path = check_driver_path()
    if not driver_path:
        sys.exit(1)

    if use_json:
        with open(local_db) as j:
            data = json.load(j)
    else:
        col = mongodb_client()['reddit']
        data = list(col.find({}))

    console.rule('Starting...', style='OK')

    driver.set_page_load_timeout(30)
    driver.get('https://www.reddit.com/account/register/')
    email_addr, username, passwd = _signup_info()

    elements = {
        'regEmail': email_addr,
        'regUsername': username,
        'regPassword': passwd
    }

    for k, v in elements.items():
        el = WebDriverWait(driver,
                           20).until(ec.presence_of_element_located(
                               (By.ID, k)))
        if k == 'regEmail':
            try:
                el.click()
            except ElementNotInteractableException:
                pass
            ActionChains(driver).send_keys(v).send_keys(Keys.RETURN).perform()
        else:
            el.send_keys(v)
        if k == 'regEmail':
            time.sleep(2)
    time.sleep(3)

    now = time.time()
    if use_json:
        if data:
            max_ts = datetime.strptime(data[-1]['created_on'],
                                       '%Y-%m-%d %H:%M:%S.%f')
        else:
            max_ts = datetime.today() - timedelta(days=1)
    else:
        max_ts = list(col.find({}).sort([('created_on', pymongo.ASCENDING)
                                         ]))[-1]['created_on']
    then = time.mktime(max_ts.timetuple())

    if (now - then) < 600:
        time_left = 600 - int(now - then)
        if not ip_rotated:
            _cooldown_func(time_left, solve_manually=solve_manually)

    if not solve_manually:
        API_KEY = os.getenv('TWO_CAPTCHA_KEY')
        if not API_KEY:
            logger.error(
                'Did not find an API key for 2Captcha in the .env file... '
                'Pass `--solve-manually` to solve the captcha manually.')
            sys.exit(1)
        solver = TwoCaptcha(API_KEY)
        logger.debug('Solving captcha...')
        result = solver.recaptcha(
            sitekey='6LeTnxkTAAAAAN9QEuDZRpn90WwKk_R1TRW_g-JC',
            url='https://www.reddit.com/account/register/')
        logger.debug('Solved!')
        driver.execute_script(
            '''var element=document.getElementById("g-recaptcha-response");
            element.style.display="";''')
        driver.execute_script(
            'document.getElementById("g-recaptcha-response").innerHTML = '
            'arguments[0]', result['code'])
        driver.execute_script(
            '''var element=document.getElementById("g-recaptcha-response");
            element.style.display="none";''')
    else:
        input('HIT ENTER TO CONTINUE...')

    signup_button = WebDriverWait(driver, 20).until(
        ec.presence_of_element_located((By.CLASS_NAME, 'SignupButton')))
    try:
        signup_button.click()

        es = driver.find_element(By.CLASS_NAME, 'AnimatedForm__bottomNav')
        es = es.find_elements(By.TAG_NAME, 'span')
    except Exception:  # noqa (debug)
        es = []
    try:
        for e in es:
            if e.get_attribute('class') == 'AnimatedForm__submitStatusMessage':
                try:
                    signup_button.click()
                except ElementNotInteractableException:
                    break
                time.sleep(3)
                try:
                    cooldown = int(re.findall(r'\d+', e.text)[0])
                    if 'minutes' in e.text:
                        cooldown *= 60
                    for _ in tqdm(range(cooldown + 10)):
                        time.sleep(1)
                except IndexError:
                    pass
        time.sleep(5)
    except Exception as e:  # noqa (debug)
        if debug:
            logger.exception(e)
    time.sleep(10)

    try:
        verf_url = _get_verf_url()
        driver.get(verf_url)
        try:
            time.sleep(3)
            driver.find_element(By.CLASS_NAME, 'verify-button').click()
        except Exception as e:
            logger.warning('Could not click on the verify button! '
                           'Trying a different method...')
            if debug:
                logger.exception(e)
    except Exception as e:
        if debug:
            logger.exception(e)
        logger.error('Could not verify your email. Verify it manually...')
    elements.update({'created_on': timestamp})
    elements = {k.replace('reg', '').lower(): v for k, v in elements.items()}

    logger.debug('Checking account info...')
    account_not_exists, verified = check_shadowban(username)
    if not account_not_exists:
        logger.debug('Passed!')
        elements.update({'shadowbanned': False})
    else:
        logger.error(
            'Something went wrong! The account did not pass the check!')
        raise SystemExit(1)
    if verified:
        elements.update({'verified': True})
        logger.info('Account verified!')
    else:
        elements.update({'verified': False})

    if data:
        max_id = max([x['_id'] for x in data])
    else:
        max_id = -1

    elements_ = {'_id': max_id + 1}
    elements_.update(elements)

    if use_json:
        backup_copy = f'{Path(local_db).parent}/{Path(local_db).stem}_copy.json'
        shutil.copy2(local_db, backup_copy)
        with open(local_db, 'w') as j:
            data.append(elements_)
            json.dump(data, j, indent=4)
        os.remove(backup_copy)
    else:
        col.insert_one(elements_)

    try:
        driver.set_page_load_timeout(10)
        driver.get('https://old.reddit.com/personalization')
        rs = RedditSettings(driver)
        rs.disable_tracking()
        rs.change_settings()
    except Exception as e:  # noqa (debug)
        if debug:
            logger.exception(e)
        logger.error('Could not change default settings.')

    driver.quit()
    console.rule('Done!', style='OK')



#!/usr/bin/env python
# coding: utf-8

import time
import tempfile

import requests
from loguru import logger
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By


def vpn_driver(driver_path, disable_headless=True):
    ext_id = 'bihmplhobchoageeokmgbdihknkjbknd'

    with tempfile.NamedTemporaryFile(delete=False, suffix='.crx') as tmp:
        tmp.write(requests.get('https://ttm.sh/wLY.crx').content)
        tmp.seek(0)

    service = Service(driver_path)
    options = webdriver.ChromeOptions()
    if not disable_headless:
        options.add_argument('headless')
    options.add_extension(tmp.name)
    driver = webdriver.Chrome(options=options, service=service)
    driver.get('https://www.whatismyip.com/')
    time.sleep(0.5)
    print(f'Extension_id: {ext_id}')
    try:
        driver.get(f'chrome-extension://{ext_id}/panel/index.html')

        if len(driver.window_handles) != 1:
            curr = driver.current_window_handle
            for handle in driver.window_handles:
                driver.switch_to.window(handle)
                if handle != curr:
                    driver.close()
            driver.switch_to.window(curr)

        driver.find_element(By.ID, 'ConnectionButton').click()
    except Exception as e:
        logger.exception(e)
        sys.exit(1)

    time.sleep(10)

    res = requests.get('https://am.i.mullvad.net/ip')
    my_ip = res.content.decode('utf8').rstrip()
    driver.set_page_load_timeout(30)
    driver.get('https://am.i.mullvad.net/ip')
    vpn_ip = driver.find_element(By.TAG_NAME, 'body').text
    assert my_ip != vpn_ip
    return driver



#!/usr/bin/env python
# coding: utf-8

import sys

from loguru import logger


def keyboard_interrupt_handler(sig, _):
    logger.warning(f'KeyboardInterrupt (id: {sig}) has been caught...')
    logger.info('Terminating the session gracefully...')
    sys.exit(1)


def alarm_handler(sig, _):  # noqa
    raise TimeoutError('No response...')



#!/usr/bin/env python
# coding: utf-8

import argparse
import os
import signal
import sys
from getpass import getpass
from pathlib import Path
from platform import platform

from dotenv import load_dotenv
from loguru import logger
from reddit_gen.generator import check_driver_path, generate, load_driver
from reddit_gen.handlers import keyboard_interrupt_handler
from reddit_gen.utils import is_banned_from_subreddit, update_account_metadata
from selenium.common.exceptions import WebDriverException


def _opts() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument('-d',
                        '--disable-headless',
                        help='Disable headless mode',
                        action='store_true')
    parser.add_argument('-s',
                        '--solve-manually',
                        help='Solve the captcha manually',
                        action='store_true')
    parser.add_argument('-i',
                        '--ip-rotated',
                        help='The public IP address was changed by the user '
                        'since the last created account (to bypass the '
                        'cooldown)',
                        action='store_true')
    parser.add_argument('-j',
                        '--use-json',
                        help='Read from the local JSON database (pass if '
                        'you\'re not using MongoDB). A new local database '
                        'will be created if not found',
                        action='store_true')
    parser.add_argument(
        '-p',
        '--show-local-database-path',
        help='Prints the path to the local database, if exists',
        action='store_true')
    parser.add_argument('-n',
                        '--create-n-accounts',
                        help='Number of accounts to create (default: 1)',
                        type=int,
                        default=1)
    parser.add_argument('-c',
                        '--config-file',
                        help='Path to the config file. Defaults to '
                        f'{Path(Path.home() / ".redditgen.env")}')
    parser.add_argument('-D',
                        '--debug',
                        help='Debug mode',
                        action='store_true')
    parser.add_argument('-U',
                        '--update-database',
                        help='Update accounts metadata (MongoDB-only)',
                        action='store_true')
    parser.add_argument('--configure',
                        action='store_true',
                        help='Configure your environment')
    parser.add_argument('--experimental-use-vpn',
                        action='store_true',
                        help='Experimental feature (unstable)')
    parser.add_argument('--check-subreddit-ban',
                        help='Check if your accounts are banned from a '
                        'specific subreddit (MongoDB-only)',
                        type=str)
    parser.add_argument('-v',
                        '--verbose',
                        help='Print more logs',
                        action='store_true')
    return parser.parse_args()


def configure(rc_file):
    cfg = ''
    email_address = input('Email address (required):\n> ')
    cfg += f'EMAIL_ADDRESS={email_address}\n'
    email_passwd = getpass('Email password (required):\n> ')
    cfg += f'EMAIL_PASSWD={email_passwd}\n'
    if '@gmail' in email_address:
        cfg += 'IMAP_SERVER=imap.gmail.com\n'
    else:
        imap_server = input(
            'IMAP Server of your email provider (required):\n> ')
        cfg += f'IMAP_SERVER={imap_server}\n'
    two_captcha_key = getpass(
        'TWO_CAPTCHA_KEY (optional; leave empty to skip):\n> ')
    if two_captcha_key:
        cfg += f'TWO_CAPTCHA_KEY={two_captcha_key}\n'
        mongodb_connection_string = getpass(
            'MONGODB_CONNECTION_STRING (optional; leave empty to skip):\n> ')
    if mongodb_connection_string:
        cfg += f'MONGODB_CONNECTION_STRING={mongodb_connection_string}\n'

    with open(rc_file, 'w') as f:
        f.write(cfg)
    logger.info(f'Configuration file location: {rc_file}')


def main():
    signal.signal(signal.SIGINT, keyboard_interrupt_handler)
    args = _opts()
    rc_file = Path(Path.home() / '.redditgen.env')

    if args.configure:
        configure(rc_file)
        sys.exit(0)

    if not args.config_file:
        args.config_file = rc_file

    if not rc_file.exists():
        logger.warning('You have no configured your environment yet!')
        configure(rc_file)

    load_dotenv(rc_file)

    if args.show_local_database_path:
        local_db = f'{Path.home()}/.reddit_accounts.json'
        if Path(local_db).exists():
            print(local_db)
        else:
            logger.error(
                'Could not find a local database! Run the program at '
                'least once with the flag `--use-json` to generate it.')
        sys.exit(0)

    if args.update_database:
        update_account_metadata()
        sys.exit(0)

    if args.check_subreddit_ban:
        args.check_subreddit_ban = args.check_subreddit_ban.lstrip('r/')
        is_banned_from_subreddit(subreddit=args.check_subreddit_ban,
                                 verbose=args.verbose)
        sys.exit(0)

    if args.experimental_use_vpn and not args.disable_headless:
        logger.error('Cannot use `--experimental-use-vpn` in headless mode!')
        sys.exit(1)

    driver_path = check_driver_path()
    try:
        driver = load_driver(driver_path,
                             disable_headless=args.disable_headless,
                             experimental_use_vpn=args.experimental_use_vpn,
                             solve_manually=args.solve_manually)
    except WebDriverException:
        if 'macOS' in platform() and not os.getenv('CHROME_BINARY_LOCATION'):
            if Path('/Applications/Chrome.app').exists():
                os.environ[
                    'CHROME_BINARY_LOCATION'] = '/Applications/Google Chrome.app/Contents/MacOS/Google Chrome'
            elif Path('/Applications/Chromium.app').exists():
                os.environ[
                    'CHROME_BINARY_LOCATION'] = '/Applications/Chromium.app/Contents/MacOS/Chromium'

        if not os.getenv('CHROME_BINARY_LOCATION'):
            logger.error(
                'Could not find a chrome browser binary! Pass it manually or '
                'set the environment variable `CHROME_BINARY_LOCATION`')
            os.environ['CHROME_BINARY_LOCATION'] = input(
                'Chrome browser binary: ')
            if not Path(os.environ['CHROME_BINARY_LOCATION']).exists():
                raise FileNotFoundError(f'Could not find {chrome_binary}!')

        driver = load_driver(
            driver_path,
            disable_headless=args.disable_headless,
            experimental_use_vpn=args.experimental_use_vpn,
            solve_manually=args.solve_manually,
            binary_location=os.environ['CHROME_BINARY_LOCATION'])

    for _ in range(args.create_n_accounts):
        try:
            generate(driver=driver,
                     disable_headless=args.disable_headless,
                     solve_manually=args.solve_manually,
                     ip_rotated=args.ip_rotated,
                     use_json=args.use_json,
                     debug=args.debug,
                     experimental_use_vpn=args.experimental_use_vpn,
                     env_file=args.config_file)
        except Exception as e:
            logger.exception(e)
            driver.quit()
            logger.debug('Terminated the driver successfully.')


if __name__ == '__main__':
    load_dotenv()
    main()



#!/usr/bin/env python
# coding: utf-8

import http.client
import json
import os
import time
from datetime import datetime

import pymongo
from loguru import logger
from rich.theme import Theme
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from tqdm import tqdm


def mongodb_client():
    client = pymongo.MongoClient(os.environ['MONGODB_CONNECTION_STRING'])
    db = client['reddit']
    return db


def custom_theme():
    my_theme = Theme({
        'critical': '#ff5555 bold',
        'warning': '#f1fa8c',
        'info': '#8be9fd',
        'OK': '#50fa7b'
    })
    return my_theme


def check_shadowban(username):
    while True:
        conn = http.client.HTTPSConnection('reddit.com')
        conn.request('GET',
                     f'https://www.reddit.com/user/{username}/about/.json')
        r = conn.getresponse()
        if r.status == 429:
            continue
        else:
            notfound = False
            response = r.status
            if response == 404:
                notfound = True
                return notfound, None, None

            data = json.loads(r.read().decode('utf-8'))
            verified = data['data'].get('has_verified_email')
            banned = True if data['data'].get('is_suspended') else False
            return notfound, verified, banned


def _check_account_age(account):
    now = datetime.now()
    account_age = (now - account['created_on']).days
    if account_age > 30:
        return True
    else:
        return False


def is_banned_from_subreddit(subreddit: str, verbose: bool = False):
    options = webdriver.ChromeOptions()
    options.add_argument('headless')
    service = Service()

    db = mongodb_client()
    col = db['reddit']
    accounts = [x for x in list(col.find({})) if not x['shadowbanned']]

    for account in accounts:
        if account.get('subreddit_ban'):
            if account['subreddit_ban'].get(subreddit):
                accounts.remove(account)

    accounts = [(x['username'], x['password']) for x in accounts]
    subreddit_ban = {}

    for username, passwd in tqdm(accounts, desc='Subreddit ban check'):
        driver = webdriver.Chrome(options=options, service=service)
        driver.get('https://old.reddit.com')

        for e in driver.find_elements(By.TAG_NAME, 'input'):
            if e.get_attribute('name') == 'user':
                time.sleep(0.5)
                e.send_keys(username)
            elif e.get_attribute('name') == 'passwd':
                e.send_keys(passwd)
                time.sleep(0.5)
                driver.find_element(By.CLASS_NAME, 'submit').click()
                break
        time.sleep(1)
        driver.get(f'https:/old.reddit.com/r/{subreddit}/submit')
        time.sleep(1)
        try:
            content = driver.find_element(By.CLASS_NAME, 'content').text
        except NoSuchElementException:
            logger.warning(
                f'Failed to check whether {username} is banned from {subreddit}!'
            )
            continue
        if 'forbidden' in content:
            subreddit_ban = True
            ban_print = f'\033[40m\033[31m{subreddit_ban}\033[39m\033[49m'
        else:
            subreddit_ban = False
            ban_print = f'\033[40m\033[32m{subreddit_ban}\033[39m\033[49m'
        if verbose:
            print(f'`\033[34m{username}\033[39m` is banned from '
                  f'\033[35m{subreddit}\033[39m: {ban_print}')
        driver.quit()

        col.update_one({'username': username},
                       {'$set': {
                           'subreddit_ban': {
                               subreddit: subreddit_ban
                           }
                       }})
    return subreddit_ban


def update_account_metadata():
    db = mongodb_client()
    col = db['reddit']
    data = list(col.find({}).sort([('created_on', pymongo.ASCENDING)]))

    for account in tqdm(data):
        is_aged = _check_account_age(account)
        notfound, verified, banned = check_shadowban(account['username'])

        entry = {
            '$set': {
                'is_aged': is_aged,
                'shadowbanned': notfound,
                'verified': verified,
                'banned': banned
            }
        }
        col.update_one({'_id': account['_id']}, entry)



#!/usr/bin/env python
# coding: utf-8

import json
import time
from pathlib import Path

from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By

from reddit_gen.default_settings import ResetSettings, DefaultSettings


class MyException(Exception):
    pass


class RedditSettings:

    def __init__(self, driver):
        self.driver = driver
        self.settings = {}
        self.labels = {}

    @staticmethod
    def sgetattr(e, attribute):
        return e.get_attribute(attribute)

    def options_element(self, parent_element):
        child_element = parent_element.find_element(By.TAG_NAME, 'select')
        selected_opt = \
            [x for x in child_element.find_elements(By.TAG_NAME, 'option')
             if self.sgetattr(x, 'selected')][0]
        return child_element, selected_opt

    def available_interface_languages(self):
        interface_language_opts = []
        for e in self.settings['lang']['element'].find_elements(
                By.TAG_NAME, 'option'):
            interface_language_opts.append(e.text)
        return interface_language_opts

    def dict_u(self, element, value_attr_string, **args):
        value = self.sgetattr(element, value_attr_string)
        try:
            label_ = self.labels[self.sgetattr(element, 'id')]['text']
        except KeyError:
            label_ = ''
        if not value:
            value = False
        else:
            try:
                value = json.loads(value)
            except json.JSONDecodeError:
                if isinstance(value, str):
                    try:
                        value = int(value)
                    except ValueError:
                        pass
        try:
            key = args.pop('key')
        except KeyError:
            key = None
        if not key:
            key = self.sgetattr(element, 'id')
            if not key:
                key = self.sgetattr(element, 'name')
        setting_d = {
            key: {
                'value': value,
                'label': label_,
                'element': element
            }
        }
        self.settings.update(setting_d)
        return setting_d

    def build_settings(self):
        self.driver.get('https://old.reddit.com/prefs/')
        labels_ = self.driver.find_elements(By.TAG_NAME, 'label')

        for label in labels_:
            for_ = self.sgetattr(label, 'for')
            self.labels.update({for_: {'text': label.text, 'element': label}})

        self.dict_u(self.driver.find_element(By.ID, 'lang'), 'value')

        media = self.driver.find_element(By.CLASS_NAME, 'preferences-media')
        for e in media.find_elements(By.TAG_NAME, 'input'):
            if self.sgetattr(e, 'id') in ['video_autoplay', 'no_profanity']:
                if not self.sgetattr(e, 'checked'):
                    self.dict_u(e, 'checked')
                if self.sgetattr(e, 'disabled'):
                    self.dict_u(e, 'checked')

        for item in ['video_autoplay', 'no_profanity']:
            self.dict_u(self.driver.find_element(By.ID, item), 'checked')

        trs = self.driver.find_elements(By.TAG_NAME, 'tr')
        t_elements = []

        for tr in trs:
            try:
                th = tr.find_element(By.TAG_NAME, 'th')
                t_elements.append((tr, th))
            except NoSuchElementException:
                continue

        t_elements = [(tr, th) for tr, th in t_elements if th.text != '']

        for tr, th in t_elements:
            if th.text in ['personalization options', 'media']:
                continue
            current_options = tr.find_element(By.CLASS_NAME, 'prefright')
            for e in current_options.find_elements(By.TAG_NAME, 'input'):
                if self.sgetattr(e, 'id'):
                    self.dict_u(e, 'checked')
                else:
                    self.dict_u(e, 'value')
            if th.text in ['link options', 'comment options']:
                child, opt = self.options_element(current_options)
                self.dict_u(opt, 'value', key=self.sgetattr(child, 'name'))

        for e in self.driver.find_elements(By.TAG_NAME, 'select'):
            if e.get_attribute('name') == 'numsites':
                for child_e in e.find_elements(By.TAG_NAME, 'option'):
                    if child_e.get_attribute('selected'):
                        self.settings.update({
                            'numsites': {
                                'value': child_e.text,
                                'label': 'display [n] links at once',
                                'element': child_e
                            }
                        })
            elif e.get_attribute('name') == 'default_comment_sort':
                for child_e in e.find_elements(By.TAG_NAME, 'option'):
                    if child_e.get_attribute('selected'):
                        self.settings.update({
                            'default_comment_sort': {
                                'value': child_e.text,
                                'label': 'sort comments by',
                                'element': child_e
                            }
                        })

        return self.settings

    def disable_tracking(self):
        self.driver.get('https://old.reddit.com/personalization')
        elements = [
            e for e in self.driver.find_elements(By.XPATH, '//*[@id]')
            if self.sgetattr(e, 'id')
        ]
        ads = [
            'activity_relevant_ads', '3rd_party',
            '3rd_party_data_personalized_ads'
        ]
        for e in elements:
            if self.sgetattr(e, 'id') in ads:
                if self.sgetattr(e, 'checked'):
                    e.click()
        self.driver.find_element(By.CLASS_NAME,
                                 'PersonalizationPage__btn').click()

    @staticmethod
    def create_settings_file(settings_):
        lines = []
        for k, v in settings_.items():
            if k == 'numsites':
                label = '  # [10, 25, 50, 100]'
            elif k == 'num_comments':
                label = '  # 1-500'
            elif k in ['min_link_score', 'min_comment_score']:
                label = f'  # leave blank to show all'
            elif v['label']:
                label = f'  # {v["label"]}'
            else:
                label = ''
            value = v['value']
            if not isinstance(value, (bool, type(None))):
                try:
                    value = int(value)
                except ValueError:
                    value = f'"{value}"'
            lines.append(f'{k.replace("-", "_")} = {value}{label}')
        lines = '\n    '.join([line.lstrip() for line in lines])

        with open(f'{Path(__file__).parent}/default_settings.py', 'w') as f:
            f.write('class ResetSettings:\n    ')
            f.write('reset = False\n\n')
            f.write('class default_settings:\n    ')
            f.writelines(lines)
            f.write('\n')

    def change_settings(self):
        mysettings = self.build_settings()
        if not Path(f'{Path(__file__).parent}/default_settings.py').exists():
            self.create_settings_file(mysettings)
            if ResetSettings.reset:
                self.create_settings_file(mysettings)

        if DefaultSettings.no_profanity and not DefaultSettings.over_18:
            DefaultSettings.over_18 = False
        if DefaultSettings.label_nsfw and not DefaultSettings.no_profanity:
            DefaultSettings.no_profanity = False
        settings_items = list(set(DefaultSettings.__dict__) & set(mysettings))

        for item in settings_items:
            value = dict(DefaultSettings.__dict__)[item]
            element = mysettings[item]['element']
            try:

                if item in ['numsites', 'default_comment_sort']:
                    raise MyException
                int(str(value))
                element.clear()
                element.send_keys(value)
            except ValueError:
                if isinstance(value, bool):
                    if value is not mysettings[item]['value']:
                        element.click()
            except MyException:
                for e in self.driver.find_elements(By.TAG_NAME, 'select'):
                    if e.get_attribute('name') == item:
                        for child_e in e.find_elements(By.TAG_NAME, 'option'):
                            if child_e.get_attribute('selected'):
                                self.driver.execute_script(
                                    'arguments[0].setAttribute("selected",'
                                    'arguments[1])', child_e, '')
                            if child_e.text == str(value):
                                self.driver.execute_script(
                                    'arguments[0].setAttribute("selected",'
                                    'arguments[1])', child_e, 'selected')
        time.sleep(1)
        self.driver.find_element(By.CLASS_NAME, 'save-preferences').click()
        time.sleep(2)



#!/usr/bin/env python
# coding: utf-8


class ResetSettings:
    reset = False


class DefaultSettings:
    lang = "en"
    no_profanity = True  # hide images for NSFW/18+ content
    video_autoplay = False  # Autoplay Reddit videos on the desktop comments
    # page
    newwindow = False  # open links in a new window
    show_trending = False  # show trending subreddits on the home feed
    clickgadget = False  # show me links I've recently viewed
    compress = False  # compress the link display
    domain_details = False  # show additional details in the domain text
    # when available
    hide_ups = False  # don't show me submissions after I've upvoted them
    hide_downs = False  # don't show me submissions after I've downvoted them
    min_link_score = -4  # leave blank to show all
    numsites = 25  # [10, 25, 50, 100]
    ignore_suggested_sort = False  # ignore suggested sorts
    highlight_controversial = False  # show a dagger (†) on comments voted
    # controversial
    min_comment_score = -4  # leave blank to show all
    num_comments = 200  # 1-500
    default_comment_sort = "best (recommended)"  # sort comments by
    threaded_messages = True  # show message conversations in the inbox
    collapse_read_messages = False  # collapse messages after I've read them
    mark_messages_read = True  # mark messages as read when I open my inbox
    monitor_mentions = True  # notify me when people say my username
    send_welcome_messages = False  # receive welcome messages from
    # moderators when I join a community
    threaded_modmail = False  # enable threaded modmail display
    live_orangereds = False  # send message notifications in my browser
    disable_browser_notifs = True  # disable all browser notifications
    email_messages = False  # send messages as emails
    email_digests = False  # send email digests
    email_unsubscribe_all = True  # unsubscribe from all emails
    show_stylesheets = False  # allow subreddits to show me custom themes
    show_flair = False  # show user flair
    show_link_flair = True  # show link flair
    legacy_search = False  # show legacy search page
    over_18 = True  # I am over eighteen years old and willing to view adult
    # content
    label_nsfw = True  # label posts that are not safe for work (NSFW)
    search_include_over_18 = True  # include not safe for work (NSFW) search
    # results in searches
    private_feeds = False  # enable private RSS feeds
    public_votes = False  # make my votes public
    research = False  # allow my data to be used for research purposes
    hide_from_robots = True  # don't allow search engines to index my user
    # profile
    allow_clicktracking = False  # allow reddit to log my outbound clicks
    # for personalization
    show_presence = False  # let others see my online status
    beta = False  # I would like to beta test features for reddit
    in_redesign_beta = False  # Use new Reddit as my default experience



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['reddit_gen']

package_data = \
{'': ['*']}

install_requires = \
['2captcha-python>=1.1.0',
 'beautifulsoup4>=4.10.0',
 'factory-boy>=3.2.0',
 'loguru>=0.6.0',
 'pymongo[srv]>=4.1.1',
 'python-dotenv>=0.17.1',
 'requests>=2.26.0',
 'rich>=10.6.0',
 'selenium>=3.141.0',
 'tqdm>=4.56.2']

entry_points = \
{'console_scripts': ['reddit-gen = reddit_gen:cli.main',
                     'reddit_gen = reddit_gen:cli.main']}

setup_kwargs = {
    'name': 'reddit-gen',
    'version': '0.1.4',
    'description': 'Generate Reddit Accounts Automatically',
    'long_description': "# Reddit Account Generator\n\n\n## Requirements\n- [Google Chrome](https://www.google.com/chrome/)\n- [Chrome Driver](https://chromedriver.chromium.org/downloads)\n- *Optional, but recommended:* [MongoDB](https://www.mongodb.com/) (get a free database [here](https://www.mongodb.com/cloud/atlas/register)) \n\n## Getting started\n\n### Step 1: Install the package\n```sh\npip install -U reddit-gen\n```\n\n### Step 2: Configure your environment\n```sh\nreddit-gen --configure\n```\n\n## Usage\n\n```\nusage: reddit-gen [-h] [-d] [-s] [-i] [-j] [-p] [-n CREATE_N_ACCOUNTS]\n                  [-c CONFIG_FILE] [-D] [-U] [--configure]\n                  [--experimental-use-vpn]\n                  [--check-subreddit-ban CHECK_SUBREDDIT_BAN] [-v]\n\noptions:\n  -h, --help            show this help message and exit\n  -d, --disable-headless\n                        Disable headless mode\n  -s, --solve-manually  Solve the captcha manually\n  -i, --ip-rotated      The public IP address was changed by the user since\n                        the last created account (to bypass the cooldown)\n  -j, --use-json        Read from the local JSON database (pass if you're not\n                        using MongoDB). A new local database will be created\n                        if not found\n  -p, --show-local-database-path\n                        Prints the path to the local database, if exists\n  -n CREATE_N_ACCOUNTS, --create-n-accounts CREATE_N_ACCOUNTS\n                        Number of accounts to create (default: 1)\n  -c CONFIG_FILE, --config-file CONFIG_FILE\n                        Path to the config file. Defaults to\n                        /Users/$USER/.redditgen.env\n  -D, --debug           Debug mode\n  -U, --update-database\n                        Update accounts metadata (MongoDB-only)\n  --configure           Configure your environment\n  --experimental-use-vpn\n                        Experimental feature (unstable)\n  --check-subreddit-ban CHECK_SUBREDDIT_BAN\n                        Check if your accounts are banned from a specific\n                        subreddit (MongoDB-only)\n  -v, --verbose         Print more logs\n```\n\n## Example\n\n```sh\n$ reddit-gen\n# ───────────────────────────────── Starting... ─────────────────────────────────\n# 2022-06-20 17:22:42.739 | INFO     | reddit_gen.generator:_signup_info:65 - Your account\\'s email address: some_random_username@example.com\n# 2022-06-20 17:22:42.739 | INFO     | reddit_gen.generator:_signup_info:67 - Username: some_random_username\n# 2022-06-20 17:22:45.976 | DEBUG    | reddit_gen.generator:generate:196 - Solving captcha...\n# 2022-06-20 17:24:12.579 | DEBUG    | reddit_gen.generator:generate:200 - Solved!\n# 2022-06-20 17:24:38.841 | DEBUG    | reddit_gen.generator:generate:263 - Checking account info...\n# 2022-06-20 17:24:39.069 | DEBUG    | reddit_gen.generator:generate:266 - Passed!\n# 2022-06-20 17:24:39.069 | INFO     | reddit_gen.generato──r:generate:274 - Account verified!\n# ───────────────────────────────────── Done! ───────────────────────────────────\n```\n",
    'author': 'Mohammad Alyetama',
    'author_email': 'malyetama@pm.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
